@@ -1,60 +1,105 @@
 package UV;
 
-our $VERSION = '1.000000_01';
+our $VERSION = '1.908_001';
 our $XS_VERSION = $VERSION;
-$VERSION = eval $VERSION;
 
 use strict;
 use warnings;
+use Carp ();
 use Exporter qw(import);
 require XSLoader;
-
 XSLoader::load('UV', $XS_VERSION);
 
-our @EXPORT_OK  = qw(
-    UV_VERSION_MAJOR UV_VERSION_MINOR UV_VERSION_PATCH UV_VERSION_IS_RELEASE
-    UV_VERSION_SUFFIX UV_VERSION_HEX
-
-    UV_E2BIG UV_EACCES UV_EADDRINUSE UV_EADDRNOTAVAIL UV_EAFNOSUPPORT UV_EAGAIN
-    UV_EAI_ADDRFAMILY UV_EAI_AGAIN UV_EAI_BADFLAGS" UV_EAI_BADHINTS
-    UV_EAI_CANCELED UV_EAI_FAIL UV_EAI_FAMILY UV_EAI_MEMORY UV_EAI_NODATA
-    UV_EAI_NONAME UV_EAI_OVERFLOW UV_EAI_PROTOCOL UV_EAI_SERVICE
-    UV_EAI_SOCKTYPE UV_EALREADY UV_EBADF UV_EBUSY UV_ECANCELED UV_ECHARSET
-    UV_ECONNABORTED UV_ECONNREFUSED UV_ECONNRESET UV_EDESTADDRREQ UV_EEXIST
-    UV_EFAULT UV_EFBIG UV_EHOSTUNREACH UV_EINTR UV_EINVAL UV_EIO UV_EISCONN
-    UV_EISDIR UV_ELOOP UV_EMFILE UV_EMSGSIZE UV_ENAMETOOLONG UV_ENETDOWN
-    UV_ENETUNREACH UV_ENFILE UV_ENOBUFS UV_ENODEV UV_ENOENT UV_ENOMEM
-    UV_ENONET UV_ENOPROTOOPT UV_ENOSPC UV_ENOSYS UV_ENOTCONN UV_ENOTDIR
-    UV_ENOTEMPTY UV_ENOTSOCK UV_ENOTSUP UV_EPERM UV_EPIPE UV_EPROTO
-    UV_EPROTONOSUPPORT UV_EPROTOTYPE UV_ERANGE UV_EROFS UV_ESHUTDOWN UV_ESPIPE
-    UV_ESRCH UV_ETIMEDOUT UV_ETXTBSY UV_EXDEV UV_UNKNOWN UV_EOF UV_ENXIO
-    UV_EMLINK
-
-    default_loop err_name hrtime strerr translate_sys_error
+our @EXPORT_OK = (
+    @UV::EXPORT_XS,
+    qw(default_loop loop err_name hrtime strerr translate_sys_error),
+    qw(check timer),
 );
 
-# make sure all sub-classes of uv_handle_t are thought of as such
-@UV::Async::ISA =
-@UV::Check::ISA =
-@UV::FSEvent::ISA =
-@UV::FSPoll::ISA =
-@UV::Idle::ISA =
-@UV::NamedPipe::ISA =
-@UV::Poll::ISA =
-@UV::Prepare::ISA =
-@UV::Process::ISA =
-@UV::Stream::ISA =
-@UV::TCP::ISA =
-@UV::Timer::ISA =
-@UV::TTY::ISA =
-@UV::UDP::ISA =
-@UV::Signal::ISA =
-@UV::File::ISA =
-    "UV::Handle";
-1;
+# _parse_args (static, private)
+sub _parse_args {
+    my $args;
+    if ( @_ == 1 && ref $_[0] ) {
+        my %copy = eval { %{ $_[0] } }; # try shallow copy
+        Carp::croak("Argument to method could not be dereferenced as a hash") if $@;
+        $args = \%copy;
+    }
+    elsif (@_==1 && !ref($_[0])) {
+        $args = {single_arg => $_[0]};
+    }
+    elsif ( @_ % 2 == 0 ) {
+        $args = {@_};
+    }
+    else {
+        Carp::croak("Method got an odd number of elements");
+    }
+    return $args;
+}
 
-# load up the default loop
-default_loop() or die 'UV: cannot initialise libUV backend.';
+sub check {
+    require UV::Check;
+    return UV::Check->new(@_);
+}
+
+sub default_loop {
+    require UV::Loop;
+    return UV::Loop->default();
+}
+
+sub idle {
+    require UV::Idle;
+    return UV::Idle->new(@_);
+}
+
+sub loop {
+    require UV::Loop;
+    return UV::Loop->default();
+}
+
+sub poll {
+    require UV::Poll;
+    return UV::Poll->new(@_);
+}
+
+sub prepare {
+    require UV::Prepare;
+    return UV::Prepare->new(@_);
+}
+
+sub signal {
+    require UV::Signal;
+    return UV::Signal->new(@_);
+}
+
+sub tcp {
+    require UV::TCP;
+    return UV::TCP->new(@_);
+}
+
+sub timer {
+    require UV::Timer;
+    return UV::Timer->new(@_);
+}
+
+sub tty {
+    require UV::TTY;
+    return UV::TTY->new(@_);
+}
+
+sub udp {
+    require UV::UDP;
+    return UV::UDP->new(@_);
+}
+
+{
+    package UV::Exception;
+
+    use overload
+        '""' => sub { $_[0]->message },
+        fallback => 1;
+}
+
+1;
 
 __END__
 
@@ -71,6 +116,7 @@ UV - Perl interface to libuv
   use warnings;
 
   use UV;
+  use UV::Loop;
 
   # hi-resolution time
   my $hi_res_time = UV::hrtime();
@@ -79,8 +125,8 @@ UV - Perl interface to libuv
   my $loop = UV::Loop->new();
 
   # default loop
-  my $loop = UV::Loop->default_loop(); # convenience constructor
-  my $loop = UV::Loop->new(1); # Tell the constructor you want the default loop
+  my $loop = UV::Loop->default_loop(); # convenience singleton constructor
+  my $loop = UV::Loop->default(); # convenience singleton constructor
 
   # run a loop with one of three options:
   # UV_RUN_DEFAULT, UV_RUN_ONCE, UV_RUN_NOWAIT
@@ -99,18 +145,6 @@ work.
 
 Event loops that work properly on all platforms. YAY!
 
-=head1 HELP NEEDED
-
-If you are a C/XS developer, I'm pleading for help. While the test cases so far
-function as expected, some design decisions I've made up to this point have
-become somewhat untenable.
-
-Please submit PRs, yell at me on IRC, email me, call me, contact me by any means
-available to you to help me fix this and get the entirety of the libuv project
-ready for Perl use.
-
-Thanks!!
-
 =head1 CONSTANTS
 
 =head2 VERSION CONSTANTS
@@ -434,6 +468,13 @@ Unknown error
 
 The following functions are available:
 
+=head2 check
+
+    my $handle = UV::check(); # uses the default loop
+    my $handle = UV::check(loop => $some_other_loop); # non-default loop
+
+Returns a new L<UV::Check> Handle object.
+
 =head2 default_loop
 
     my $loop = UV::default_loop();
@@ -467,7 +508,51 @@ never be called.
 
     my $uint64_t = UV::hrtime();
 
-Get the current Hi-Res time (C<uint64_t>).
+Get the current Hi-Res time; a value given in nanoseconds since some arbitrary
+point in the past. On 64bit-capable perls this will be represented by an
+integer with full precision. On perls unable to represent a 64bit integer this
+will be given as a floating-point value so may lose some precision if the
+value is large enough.
+
+=head2 idle
+
+    my $handle = UV::idle(); # uses the default loop
+    my $handle = UV::idle(loop => $some_other_loop); # non-default loop
+
+Returns a new L<UV::Idle> Handle object.
+
+=head2 loop
+
+    my $loop = UV::loop();
+    # You can also get it with the UV::Loop methods below:
+    my $loop = UV::Loop->default_loop();
+    my $loop = UV::Loop->default();
+
+Returns the default loop (which is a singleton object). This module already
+creates the default loop and you get access to it with this method.
+
+=head2 poll
+
+    my $handle = UV::poll(); # uses the default loop
+    my $handle = UV::poll(loop => $some_other_loop); # non-default loop
+
+Returns a new L<UV::Poll> Handle object.
+
+=head2 prepare
+
+    my $handle = UV::prepare(); # uses the default loop
+    my $handle = UV::prepare(loop => $some_other_loop); # non-default loop
+
+Returns a new L<UV::Prepare> Handle object.
+
+=head2 signal
+
+    my $handle = UV::signal(POSIX::SIGHUP); # uses the default loop
+
+    my $handle = UV::signal(loop => $some_other_loop, signal => POSIX::SIGHUP);
+        # non-default loop
+
+Returns a new L<UV::Signal> Handle object.
 
 =head2 strerror
 
@@ -485,6 +570,31 @@ number will imply an error.
 When a function which takes a callback returns an error, the callback will
 never be called.
 
+=head2 tcp
+
+    my $tcp = UV::tcp();
+
+Returns a new L<UV::TCP> object.
+
+=head2 timer
+
+    my $timer = UV::timer(); # uses the default loop
+    my $timer = UV::timer(loop => $some_other_loop); # non-default loop
+
+Returns a new L<UV::Timer> object.
+
+=head2 tty
+
+    my $tty = UV::tty(fd => 0);
+
+Returns a new L<UV::TTY> object.
+
+=head2 udp
+
+    my $udp = UV::udp();
+
+Returns a new L<UV::UDP> object.
+
 =head2 version
 
     my $int = UV::version();
@@ -503,6 +613,37 @@ The L<version_string|http://docs.libuv.org/en/v1.x/version.html#c.uv_version_str
 function returns the libuv version number as a string. For non-release versions
 the version suffix is included.
 
+=head1 EXCEPTIONS
+
+If any call to F<libuv> fails, an exception will be thrown. The exception will
+be a blessed object having a C<code> method which returns the numerical error
+code (which can be compared to one of the C<UV::UV_E*> error constants), and a
+C<message> method which returns a human-readable string describing the failure.
+
+    try { ... }
+    catch my $e {
+        if(blessed $e and $e->isa("UV::Exception")) {
+            print "The failure was ", $e->message, " of code ", $e->code;
+        }
+    }
+
+The exception class provides stringify overload to call the C<message> method,
+so the normal Perl behaviour of just printing the exception will print the
+message from it, as expected.
+
+Exceptions are blessed into a subclass of C<UV::Exception> named after the
+type of the failure code. This allows type-based testing of error types.
+
+    try { ... }
+    catch my $e {
+        if(blessed $e and $e->isa("UV::Exception::ECANCELED") {
+            # ignore
+        }
+        else ...
+    }
+
+=cut
+
 =head1 AUTHOR
 
 Chase Whitener <F<capoeirab@cpan.org>>
