@@ -1,13 +1,30 @@
 package UV::Check;
 
-our $VERSION = '1.000000_01';
-$VERSION = eval $VERSION;
+our $VERSION = '1.908_001';
 
 use strict;
 use warnings;
-
+use Carp ();
 use parent 'UV::Handle';
 
+sub start {
+    my $self = shift;
+    if (@_) {
+        $self->on('check', shift);
+    }
+    my $res;
+    my $err = do { #catch
+        local $@;
+        eval {
+            $res = $self->_start();
+            1;
+        }; #try
+        $@;
+    };
+    Carp::croak($err) if $err; # throw
+    return $res;
+}
+
 1;
 
 __END__
@@ -31,7 +48,11 @@ UV::Check - Check handles in libuv
 
   # Use a different loop
   my $loop = UV::Loop->new(); # non-default loop
-  my $check = UV::Check->new($loop);
+  my $check = UV::Check->new(
+    loop => $loop,
+    on_close => sub {say "close!"},
+    on_check => sub {say "check!"},
+  );
 
   # setup the check callback:
   $check->on(check => sub {say "We're CHECKING!!!"});
@@ -80,7 +101,11 @@ following extra methods available.
 
     my $check = UV::Check->new();
     # Or tell it what loop to initialize against
-    my $check = UV::Check->new($loop);
+    my $check = UV::Check->new(
+        loop => $loop,
+        on_close => sub {say "close!"},
+        on_check => sub {say "check!"},
+    );
 
 This constructor method creates a new L<UV::Check> object and
 L<initializes|http://docs.libuv.org/en/v1.x/check.html#c.uv_check_init> the
@@ -102,6 +127,8 @@ L<UV::Loop/"default_loop"> is assumed.
 The L<start|http://docs.libuv.org/en/v1.x/check.html#c.uv_check_start> method
 starts the check handle.
 
+Returns the C<$check> instance itself.
+
 =head2 stop
 
     $check->stop();
