@@ -1,19 +1,50 @@
 package UV::Handle;
 
-our $VERSION = '1.000000_01';
-$VERSION = eval $VERSION;
+our $VERSION = '1.908_001';
 
 use strict;
 use warnings;
+use Carp ();
 use Exporter qw(import);
+use UV ();
+use UV::Loop ();
 
-use UV;
+sub _new_args {
+    my ($class, $args) = @ _;
+    my $loop = delete $args->{loop} // UV::Loop->default;
+    return ($loop);
+}
 
-our @EXPORT_OK  = qw(
-    UV_ASYNC UV_CHECK UV_FS_EVENT UV_FS_POLL
-    UV_IDLE UV_NAMED_PIPE UV_POLL UV_PREPARE UV_PROCESS
-    UV_STREAM UV_TCP UV_TIMER UV_TTY UV_UDP UV_SIGNAL UV_FILE
-);
+sub new {
+    my $class = shift;
+    my %args = @_;
+
+    my $self = $class->_new($class->_new_args(\%args));
+    $self->on(($_ =~ m/^on_(.*)/)[0] => delete $args{$_}) for grep { m/^on_/ } keys %args;
+
+    if(%args) {
+        my $code;
+        $code = $self->can("_set_$_") and $self->$code(delete $args{$_}) for keys %args;
+        die "TODO: more args @{[ keys %args ]}" if keys %args;
+    }
+
+    return $self;
+}
+
+sub on {
+    my $self = shift;
+    my $method = "_on_" . shift;
+    return $self->$method( @_ );
+}
+
+sub close {
+    my $self = shift;
+    $self->on('close', @_) if @_;
+
+    return if $self->closed || $self->closing;
+    $self->stop if $self->can('stop');
+    $self->_close;
+}
 
 1;
 
@@ -91,19 +122,6 @@ from those sub-classes.
 
 L<UV::Handle> makes the following extra events available.
 
-=head2 alloc
-
-    $handle->on("alloc", sub { say "We are allocating!"});
-    $handle->on("alloc", sub {
-        # the handle instance this event fired on and the buffer size in use
-        my ($invocant, $buffer_size) = @_;
-        say "A buffer of size $buffer_size was just allocated for us!";
-    });
-
-The L<alloc|http://docs.libuv.org/en/v1.x/handle.html#c.uv_alloc_cb> callback
-fires when a C<< $handle->read_start() >> or C<< $handle->recv_start() >>
-method gets called.
-
 =head2 close
 
     $handle->on("close", sub { say "We are closing!"});
@@ -128,10 +146,8 @@ L<UV::Handle> implements the following attributes.
     $handle = $handle->data(undef);
     my $data = $handle->data();
 
-The L<data|http://docs.libuv.org/en/v1.x/handle.html#c.uv_handle_t.data>
-attribute allows you to store some information along with your L<UV::Handle>
-object. Since libuv does not make use of this attribute in any way, you're free
-to use it for your own purposes.
+The C<data> attribute allows you to store some information along with your
+L<UV::Handle> object that you can for your own purposes.
 
 =head2 loop
 
@@ -142,18 +158,6 @@ The L<loop|http://docs.libuv.org/en/v1.x/handle.html#c.uv_handle_t.loop>
 attribute is a B<read-only> attribute that returns the L<UV::Loop> object this
 handle was initialized with.
 
-=head2 type
-
-    # read-only attribute
-    my $int = $handle->type();
-    if ($int == UV::Handle::UV_TIMER) {
-        say "This handle is a timer";
-    }
-
-The L<type|http://docs.libuv.org/en/v1.x/handle.html#c.uv_handle_t.type>
-attribute is a B<read-only> attribute that returns the corresponding libuv
-handle type constant.
-
 =head1 METHODS
 
 L<UV::Handle> makes the following methods available.
@@ -208,6 +212,13 @@ In-progress requests, like C<< $handle->connect() >> or C<< $handle->write >>,
 are canceled and have their callbacks called asynchronously with
 C<< status = UV::UV_ECANCELED >>.
 
+=head2 closed
+
+    # are we officially closed?
+    my $int = $handle->closed();
+
+B<Read-only> method to let us know if the handle is closed.
+
 =head2 closing
 
     my $int = $handle->closing();
@@ -218,29 +229,6 @@ method returns non-zero if the handle is closing or closed, zero otherwise.
 B<* Note:> This function should only be used between the initialization of the
 handle and the arrival of the C<close> callback.
 
-=head2 has_ref
-
-    my $int = $handle->has_ref();
-
-The L<has_ref|http://docs.libuv.org/en/v1.x/handle.html#c.uv_has_ref>
-method returns non-zero if the handle is referenced, zero otherwise.
-
-See L<Reference Counting|http://docs.libuv.org/en/v1.x/handle.html#refcount>.
-
-=head2 is_active
-
-    # simply a synonym for ->active()
-    my $int = $handle->is_active();
-
-A synonym for L<UV::Handle/"active">.
-
-=head2 is_closing
-
-    # simply a synonym for ->closing()
-    my $int = $handle->is_closing();
-
-A synonym for L<UV::Handle/"closing">.
-
 =head2 on
 
     # set a close event callback to print the handle's data attribute
@@ -257,28 +245,6 @@ A synonym for L<UV::Handle/"closing">.
 The C<on> method allows you to subscribe to L<UV::Handle/"EVENTS"> emitted by
 any UV::Handle or subclass.
 
-=head2 ref
-
-    $handle->ref();
-
-The L<ref|http://docs.libuv.org/en/v1.x/handle.html#c.uv_ref>
-method references the given handle. References are idempotent, that is, if a
-handle is already referenced, then calling this function again will have no
-effect.
-
-See L<Reference Counting|http://docs.libuv.org/en/v1.x/handle.html#refcount>.
-
-=head2 unref
-
-    $handle->unref();
-
-The L<unref|http://docs.libuv.org/en/v1.x/handle.html#c.uv_unref>
-method un-references the given handle. References are idempotent, that is, if a
-handle is not referenced, then calling this function again will have no
-effect.
-
-See L<Reference Counting|http://docs.libuv.org/en/v1.x/handle.html#refcount>.
-
 
 =head1 AUTHOR
 
