@@ -1,13 +1,31 @@
 package UV::Idle;
 
-our $VERSION = '1.000000_01';
-$VERSION = eval $VERSION;
+our $VERSION = '1.908_001';
 
 use strict;
 use warnings;
-
 use parent 'UV::Handle';
 
+use Carp ();
+
+sub start {
+    my $self = shift;
+    if (@_) {
+        $self->on('idle', shift);
+    }
+    my $res;
+    my $err = do { #catch
+        local $@;
+        eval {
+            $res = $self->_start();
+            1;
+        }; #try
+        $@;
+    };
+    Carp::croak($err) if $err; # throw
+    return $res;
+}
+
 1;
 
 __END__
@@ -31,7 +49,11 @@ UV::Idle - Idle handles in libuv
 
   # Use a different loop
   my $loop = UV::Loop->new(); # non-default loop
-  my $idle = UV::Idle->new($loop);
+  my $idle = UV::Idle->new(
+    loop => $loop,
+    on_close => sub {say "close!"},
+    on_idle => sub {say "idle!"},
+  );
 
   # setup the idle callback:
   $idle->on(idle => sub {say "We're IDLING!!!"});
@@ -88,7 +110,11 @@ following extra methods available.
 
     my $idle = UV::Idle->new();
     # Or tell it what loop to initialize against
-    my $idle = UV::Idle->new($loop);
+    my $idle = UV::Idle->new(
+        loop => $loop,
+        on_close => sub {say "close!"},
+        on_idle => sub {say "idle!"},
+    );
 
 This constructor method creates a new L<UV::Idle> object and
 L<initializes|http://docs.libuv.org/en/v1.x/idle.html#c.uv_idle_init> the
@@ -110,6 +136,8 @@ L<UV::Loop/"default_loop"> is assumed.
 The L<start|http://docs.libuv.org/en/v1.x/idle.html#c.uv_idle_start> method
 starts the handle.
 
+Returns the C<$idle> instance itself.
+
 =head2 stop
 
     $idle->stop();
