@@ -1,17 +1,67 @@
 package UV::Loop;
 
-our $VERSION = '1.000000_01';
-$VERSION = eval $VERSION;
+our $VERSION = '1.908_001';
 
 use strict;
 use warnings;
+
+use Carp ();
 use Exporter qw(import);
+use Scalar::Util ();
+use UV ();
 
-use UV;
+use constant DEBUG => $ENV{PERL_UV_DEBUG};
 
-our @EXPORT_OK  = qw(
-    UV_RUN_DEFAULT UV_RUN_ONCE UV_RUN_NOWAIT UV_LOOP_BLOCK_SIGNAL SIGPROF
-);
+our @EXPORT_OK = (@UV::Loop::EXPORT_XS,);
+my $default_loop;
+
+# simple function to ensure we've been given a UV::Loop
+# this is useful in new Handle construction
+sub _is_a_loop {
+    my $loop = shift;
+    return undef unless $loop;
+    return undef unless ref($loop) && Scalar::Util::blessed($loop);
+    return undef unless $loop->isa('UV::Loop');
+    return 1;
+}
+
+sub new {
+    my $class = shift;
+    print STDERR "UV::Loop->new() called\n" if DEBUG;
+    my $args = UV::_parse_args(@_);
+
+    my $self = $class->_new($args->{_default} // 0);
+
+    print STDERR "UV::Loop->new() done\n" if DEBUG;
+    return $self;
+}
+
+# Return the singleton uv_default_loop
+sub default {
+    print STDERR "loop default() singleton called\n" if DEBUG;
+    my $class = shift;
+    if (defined($default_loop)) {
+        print STDERR "loop default() returning already stored default loop\n" if DEBUG;
+        return $default_loop;
+    }
+    print STDERR "loop default() We don't have a default. Let's create one!\n" if DEBUG;
+    $default_loop = $class->new(@_, _default => 1);
+    print STDERR "loop default() returning newly created and stored default loop\n" if DEBUG;
+    return $default_loop;
+}
+
+sub default_loop { return shift->default(); }
+
+sub walk {
+    Carp::croak "TODO: UV::Loop->walk";
+}
+
+sub getaddrinfo {
+    my $self = shift;
+    my ($args, $cb) = @_;
+
+    $self->_getaddrinfo(@{$args}{qw( node service flags family socktype protocol )}, $cb);
+}
 
 1;
 
@@ -29,14 +79,14 @@ UV::Loop - Looping with libuv
   use strict;
   use warnings;
 
-  use UV;
+  use UV::Loop ();
 
-  # A new loop
+  # A new, non-default loop
   my $loop = UV::Loop->new();
 
-  # default loop
-  my $loop = UV::Loop->default_loop(); # convenience constructor
-  my $loop = UV::Loop->new(1); # Tell the constructor you want the default loop
+  # A new default loop instance (Singleton)
+  $loop = UV::Loop->default_loop(); # singleton constructor
+  $loop = UV::Loop->default(); # singleton constructor
 
   # run a loop with one of three options:
   # UV_RUN_DEFAULT, UV_RUN_ONCE, UV_RUN_NOWAIT
@@ -72,6 +122,7 @@ Event loops that work properly on all platforms. YAY!
 
 =head3 UV_LOOP_BLOCK_SIGNAL
 
+
 =head1 METHODS
 
 L<UV::Loop> makes the following methods available.
@@ -79,7 +130,6 @@ L<UV::Loop> makes the following methods available.
 =head2 new
 
     my $loop = UV::Loop->new();
-    my $default_loop = UV::Loop->new(1);
     my $default_loop = UV::Loop->default_loop();
     my $default_loop = UV::Loop->default();
 
@@ -120,16 +170,6 @@ The L<backend_timeout|http://docs.libuv.org/en/v1.x/loop.html#c.uv_backend_timeo
 method returns the poll timeout. The return value is in milliseconds, or C<-1>
 for no timeout.
 
-=head2 close
-
-    $loop->close();
-
-The L<close|http://docs.libuv.org/en/v1.x/loop.html#c.uv_loop_close> method
-releases all internal loop resources. Call this method only when the loop has
-finished executing and all open handles and requests have been closed, or it
-will return C<UV::UV_EBUSY>. After this method returns, the user can free the
-memory allocated for the loop.
-
 =head2 configure
 
     my $int = $loop->configure();
@@ -138,10 +178,6 @@ The L<configure|http://docs.libuv.org/en/v1.x/loop.html#c.uv_loop_configure>
 method sets additional loop options. You should normally call this before the
 first call to L<UV::Loop/"run"> unless mentioned otherwise.
 
-Returns C<0> on success or a C<UV/"CONSTANTS"> error code on failure. Be
-prepared to handle C<UV::UV_ENOSYS>; it means the loop option is not supported
-by the platform.
-
 Supported options:
 
 =over
@@ -159,29 +195,24 @@ signals will fail with C<UV::UV_EINVAL>.
 
 =head2 default
 
-    # simply a synonym for ->new(1)
+    # this is a singleton constructor. you'll get the same instance each time
     my $default_loop = UV::Loop->default();
 
-A synonym for the L<UV::Loop/"new"> constructor.
+A singleton method to get the default loop instance.
 
 =head2 default_loop
 
-    # simply a synonym for ->new(1)
+    # this is a singleton constructor. you'll get the same instance each time
     my $default_loop = UV::Loop->default_loop();
 
-A synonym for the L<UV::Loop/"new"> constructor.
+A singleton method to get the default loop instance.
 
-=head2 loop_alive
+=head2 is_default
 
-    my $int = $loop->loop_alive();
+    # lets us know if this loop is the default loop for this context
+    my $bool = $loop->is_default();
 
-This is just a synonym for L<UV::Loop/"alive">.
-
-=head2 loop_configure
-
-    my $int = $loop->configure(UV::Loop::UV_LOOP_BLOCK_SIGNAL, SIGPROF);
-
-This is just a synonym for L<UV::Loop/"configure">.
+A read-only method to let us know if we're dealing with the default loop.
 
 =head2 now
 
@@ -259,28 +290,60 @@ subjective but probably on the order of a millisecond or more.
 
 =head2 walk
 
-    # although you can do it, calling ->walk() without a callback is pretty
-    # useless.
-    # call with no callback
-    $loop->walk();
-    $loop->walk(undef);
-    $loop->walk(sub {});
+The L<walk|http://docs.libuv.org/en/v1.x/loop.html#c.uv_walk> method is
+currently not implemented. See also
 
-    # instead, let's walk the loop and cleanup any handles attached and then
-    # completely close the loop.
-    $loop->walk(sub {
-        my $handle = shift;
-        # check to make sure the handle can stop
-        $handle->stop() if $handle->can('stop');
-        $handle->close() unless $handle->closing();
-        $loop->run(UV::Loop::UV_RUN_DEFAULT);
-        $loop->close();
-    });
+L<https://github.com/p5-UV/p5-UV/issues/33>
 
-The L<walk|http://docs.libuv.org/en/v1.x/loop.html#c.uv_walk> method will
-C<walk> the list of handles and fire off the callback supplied.
+=head2 getaddrinfo
 
-This is an excellent way to ensure your loop is completely cleaned up.
+    $req = $loop->getaddrinfo($args, $callback);
+
+        $callback->($status, @results)
+
+The L<getaddrinfo|http://docs.libuv.org/en/v1.x/dns.html#c.uv_getaddrinfo>
+method performs an asynchronous name lookup, turning a hostname and/or service
+name into a set of socket addresses suitable for C<connect()> or C<bind()>.
+
+The arguments passed by hash reference must include at least one of C<node>
+and C<service>, giving names of the entity to be looked up. Optional numerical
+parameters C<flags>, C<family>, C<socktype> and C<protocol> will be passed as
+hints if given.
+
+The method returns a L<UV::Req> instance representing the pending request. The
+caller does not need to hold a reference to it, but it may be used to cancel
+the request if so.
+
+When complete, the callback will be invoked with a status code indicating
+success or failure, and a list of result objects. Each value in the result
+list will have methods C<family>, C<socktype> and C<protocol> returning
+integers, and C<addr> and C<canonname> returning a string.
+
+    $result->family
+    $result->socktype
+    $result->protocol
+    $result->addr
+    $result->canonname
+
+The C<canonname> field will only be set on the first result, and only if the
+C<AI_CANONNAME> flag was included in the request.
+
+=head2 getnameinfo
+
+    $req = $loop->getnameinfo($addr, $flags)
+
+        $callback->($status, $hostname, $service)
+
+The L<getnameinfo|http://docs.libuv.org/en/v1.x/dns.html#c.uv_getnameinfo>
+method performs an asynchronous reverse name lookup, turning a socket address
+into a human-readable host and service name.
+
+The method returns a L<UV::Req> instance representing the pending request. The
+caller does not need to hold a reference to it, but it may be used to cancel
+the request if so.
+
+When complete, the callback will be invoked with a status code indicating
+success or failure, and the resolved host and service names.
 
 
 =head1 AUTHOR
