@@ -1,17 +1,60 @@
 package UV::Poll;
 
-our $VERSION = '1.000000_01';
-$VERSION = eval $VERSION;
+our $VERSION = '1.908_001';
 
 use strict;
 use warnings;
-
+use Carp ();
+use Exporter qw(import);
 use parent 'UV::Handle';
-our @EXPORT_OK  = qw(
-    UV_READABLE UV_WRITABLE UV_DISCONNECT UV_PRIORITIZED
-);
 
+our @EXPORT_OK = (@UV::Poll::EXPORT_XS,);
 
+sub _new_args {
+    my ($class, $args) = @_;
+    my ($fd, $is_socket);
+    for my $key (qw(fd fh socket single_arg)) {
+        next if !defined($fd = delete $args->{$key});
+
+        $is_socket = $key eq "socket";
+        if (CORE::ref($fd) eq 'GLOB' || CORE::ref($fd) =~ /^IO::Socket/) {
+            $fd = fileno($fd);
+            last;
+        }
+        elsif (!CORE::ref($fd) && $fd =~ /\A[0-9]+\z/) {
+            last;
+        }
+        else {
+            $fd = undef;
+        }
+    }
+    unless (defined($fd)) {
+        Carp::croak("No file or socket descriptor provided");
+    }
+    return ($class->SUPER::_new_args($args), $fd, $is_socket);
+}
+
+sub start {
+    my $self = shift;
+    Carp::croak("Can't start a closed handle") if $self->closed();
+
+    my $events = shift(@_) || UV::Poll::UV_READABLE;
+
+    if (@_) {
+        $self->on('poll', shift);
+    }
+    my $res;
+    my $err = do { #catch
+        local $@;
+        eval {
+            $res = $self->_start($events);
+            1;
+        }; #try
+        $@;
+    };
+    Carp::croak($err) if $err; # throw
+    return $res;
+}
 
 1;
 
@@ -33,11 +76,25 @@ UV::Poll - Poll handles in libuv
 
   # assume we have a file/IO handle from somewhere
   # A new handle will be initialized against the default loop
-  my $poll = UV::Poll->new(fileno($handle));
+  my $poll = UV::Poll->new(fd => fileno($handle));
 
   # Use a different loop
   my $loop = UV::Loop->new(); # non-default loop
-  my $poll = UV::Poll->new(fileno($handle), $loop);
+  my $poll = UV::Poll->new(
+    fd => fileno($handle),
+    loop => $loop,
+    on_close => sub {say "close!"},
+    on_poll => sub {say "poll!"},
+  );
+
+  # Create a new poll on a socket handle
+  my $socket = IO::Socket::INET->new(Type => SOCK_STREAM);
+  my $poll = UV::Poll->new(
+    socket => 1,
+    fd => fileno($socket),
+    on_close => sub {say "close!"},
+    on_poll => sub {say "poll!"},
+  );
 
   # setup the handle's callback:
   $poll->on(poll => sub {"We're prepared!!!"});
@@ -110,7 +167,7 @@ following extra events available.
         say "We are here!";
     });
     my $count = 0;
-    $poll->on(prepare => sub {
+    $poll->on(poll => sub {
         my $invocant = shift; # the handle instance this event fired on
         if (++$count > 2) {
             say "We've been called twice. stopping!";
@@ -129,36 +186,42 @@ following extra methods available.
 
 =head2 new
 
-    my $poll = UV::Poll->new(fileno($some_handle));
-    # Or tell it what loop to initialize against
-    my $poll = UV::Poll->new(fileno($some_handle), $loop);
-
-This constructor method creates a new L<UV::Poll> object and
-L<initializes|http://docs.libuv.org/en/v1.x/poll.html#c.uv_poll_init> the
-handle with the given L<UV::Loop> to poll for a file descriptor. If no
-L<UV::Loop> is provided, then the L<UV::Loop/"default_loop"> is assumed.
-
-B<* Note:> As of libuv v1.2.2: the file descriptor is set to non-blocking mode.
-
-=head2 new_socket
-
     use IO::Socket::INET;
-    use UV;
+    use UV ();
+    use UV::Loop ();
     use UV::Poll qw(UV_READABLE UV_WRITABLE);
 
+    my $poll = UV::Poll->new(fd => fileno($some_handle));
+    # Or tell it what loop to initialize against
+    my $poll = UV::Poll->new(fd => fileno($some_handle), loop => $loop);
+
     my $socket = IO::Socket::INET->new(Type => SOCK_STREAM);
 
-    my $poll = UV::Poll->new_socket(fileno($socket));
-    my $poll = UV::Poll->new_socket(fileno($socket), $some_loop); # or another loop
+    my $poll = UV::Poll->new(socket => $socket);
+    # or another loop
+    my $poll = UV::Poll->new(
+        socket => $socket,
+        loop => $some_loop,
+        on_close => sub {say "close!"},
+        on_poll => sub {say "poll!"},
+    );
 
     $poll->run(UV_READABLE | UV_WRITABLE, sub { ... });
 
-This constructor method creates a new L<UV::Poll> object and
-L<initializes|http://docs.libuv.org/en/v1.x/poll.html#c.uv_poll_init_socket> the
-handle with the given L<UV::Loop> to poll for a socket handle. If no
-L<UV::Loop> is provided, then the L<UV::Loop/"default_loop"> is assumed.
+This constructor method creates a new L<UV::Poll> object instance. It
+initializes the handle with either the given L<UV::Loop> or the
+L<default loop|UV::Loop/"default">.
 
-B<* Note:> As of libuv v1.2.2: the file socket is set to non-blocking mode.
+Then initialization happens with
+L<init|http://docs.libuv.org/en/v1.x/poll.html#c.uv_poll_init> or
+L<init_socket|http://docs.libuv.org/en/v1.x/poll.html#c.uv_poll_init_socket>.
+
+A handle specified by the C<socket> argument is initialised via
+C<uv_poll_init_socket>, a distinction which matters to F<MSWin32>. Code which
+wishes to be portable should remember to use this form so it will work
+correctly on that OS, even if the code works fine on others without it.
+
+B<* Note:> As of libuv v1.2.2: the file descriptor is set to non-blocking mode.
 
 =head2 start
 
@@ -167,6 +230,9 @@ B<* Note:> As of libuv v1.2.2: the file socket is set to non-blocking mode.
     # use whatever callback was supplied with ->on(poll => sub {...})
     $poll->start();
 
+    # Pass events
+    $poll->start(UV_READABLE | UV_WRITABLE);
+
     # pass a callback for the "idle" event
     $poll->start(UV_READABLE, sub {say "yay"});
     # providing the callback above completely overrides any callback previously
@@ -199,6 +265,8 @@ fine. Doing so will update the events mask that is being watched for.
 B<* Note:> Though C<UV_DISCONNECT> can be set, it is unsupported on AIX and as
 such will not be set on the C<$events> field in the callback.
 
+Returns the C<$poll> instance itself.
+
 =head2 stop
 
     $poll->stop();
