@@ -1,13 +1,30 @@
 package UV::Prepare;
 
-our $VERSION = '1.000000_01';
-$VERSION = eval $VERSION;
+our $VERSION = '1.908_001';
 
 use strict;
 use warnings;
-
+use Carp ();
 use parent 'UV::Handle';
 
+sub start {
+    my $self = shift;
+    if (@_) {
+        $self->on('prepare', shift);
+    }
+    my $res;
+    my $err = do { #catch
+        local $@;
+        eval {
+            $res = $self->_start();
+            1;
+        }; #try
+        $@;
+    };
+    Carp::croak($err) if $err; # throw
+    return $res;
+}
+
 1;
 
 __END__
@@ -31,7 +48,11 @@ UV::Prepare - Prepare handles in libuv
 
   # Use a different loop
   my $loop = UV::Loop->new(); # non-default loop
-  my $prepare = UV::Prepare->new($loop);
+  my $prepare = UV::Prepare->new(
+    loop => $loop,
+    on_close => sub {say "close!"},
+    on_prepare => sub {say "prepare!"},
+  );
 
   # setup the handle's callback:
   $prepare->on(prepare => sub {say "We're prepared!!!"});
@@ -57,7 +78,7 @@ before polling for i/o.
 L<UV::Prepare> inherits all events from L<UV::Handle> and also makes the
 following extra events available.
 
-=head2 timer
+=head2 prepare
 
     $prepare->on(prepare => sub { my $invocant = shift; say "We are here!"});
     my $count = 0;
@@ -82,7 +103,11 @@ following extra methods available.
 
     my $prepare = UV::Prepare->new();
     # Or tell it what loop to initialize against
-    my $prepare = UV::Prepare->new($loop);
+    my $prepare = UV::Prepare->new(
+        loop => $loop,
+        on_close => sub {say "close!"},
+        on_prepare => sub {say "prepare!"},
+    );
 
 This constructor method creates a new L<UV::Prepare> object and
 L<initializes|http://docs.libuv.org/en/v1.x/prepare.html#c.uv_prepare_init> the
@@ -104,6 +129,8 @@ L<UV::Loop/"default_loop"> is assumed.
 The L<start|http://docs.libuv.org/en/v1.x/prepare.html#c.uv_prepare_start> method
 starts the handle.
 
+Returns the C<$prepare> instance itself.
+
 =head2 stop
 
     $prepare->stop();
