@@ -1,13 +1,42 @@
 package UV::Timer;
 
-our $VERSION = '1.000000_01';
-$VERSION = eval $VERSION;
+our $VERSION = '1.908_001';
 
 use strict;
 use warnings;
-
+use Carp ();
+use Exporter qw(import);
 use parent 'UV::Handle';
 
+our @EXPORT_OK = (@UV::Timer::EXPORT_XS,);
+
+sub repeat {
+    my $self = shift;
+    return $self->_get_repeat() unless (@_);
+    $self->_set_repeat(@_);
+    return $self;
+}
+
+sub start {
+    my $self = shift;
+    Carp::croak("Can't start a closed handle") if ($self->closed()) ;
+    my $timeout = shift(@_) || 0;
+    my $repeat = shift(@_) || 0;
+    if (@_) {
+        $self->on('timer', shift);
+    }
+    my $res;
+    my $err = do { #catch
+        local $@;
+        eval {
+            $res = $self->_start($timeout, $repeat);
+            1;
+        }; #try
+        $@;
+    };
+    Carp::croak($err) if $err; # throw
+    return $res;
+}
 1;
 
 __END__
@@ -31,7 +60,11 @@ UV::Timer - Timers in libuv
 
   # Use a different loop
   my $loop = UV::Loop->new(); # non-default loop
-  my $timer = UV::Timer->new($loop);
+  my $timer = UV::Timer->new(
+    loop => $loop,
+    on_close => sub {say "close!"},
+    on_timer => sub {say "timer!"},
+  );
 
   # setup the timer callback:
   $timer->on("timer", sub {say "We're TIMING!!!"});
@@ -80,7 +113,11 @@ following extra methods available.
 
     my $timer = UV::Timer->new();
     # Or tell it what loop to initialize against
-    my $timer = UV::Timer->new($loop);
+    my $timer = UV::Timer->new(
+        loop => $loop,
+        on_close => sub {say "close!"},
+        on_timer => sub {say "timer!"},
+    );
 
 This constructor method creates a new L<UV::Timer> object and
 L<initializes|http://docs.libuv.org/en/v1.x/timer.html#c.uv_timer_init> the
@@ -95,19 +132,16 @@ The L<again|http://docs.libuv.org/en/v1.x/timer.html#c.uv_timer_again> method
 stops the timer, and if it is repeating, restarts it using the repeat value as
 the timeout.  If the timer has never been started, it returns C<UV::UV_EINVAL>.
 
-=head2 get_repeat
+=head2 repeat
 
-    my $uint64_t = $timer->get_repeat();
+    my $uint64_t = $timer->repeat();
+    $timer = $timer->repeat(12345); # method chaining
 
-The L<get_repeat|http://docs.libuv.org/en/v1.x/timer.html#c.uv_timer_get_repeat>
-method returns the timer's repeat value.
+The repeat method returns the timer's repeat value via:  L<get_repeat|http://docs.libuv.org/en/v1.x/timer.html#c.uv_timer_get_repeat>
+or sets the timer's repeat value via
+L<set repeat|http://docs.libuv.org/en/v1.x/timer.html#c.uv_timer_set_repeat>.
 
-=head2 set_repeat
-
-    $timer->set_repeat(12345);
-
-The L<set repeat|http://docs.libuv.org/en/v1.x/timer.html#c.uv_timer_set_repeat>
-method sets the repeat interval in I<milliseconds>. The timer will be scheduled
+The repeat value is set in I<milliseconds>. The timer will be scheduled
 to run on the given interval, regardless of the callback execution duration,
 and will follow normal timer semantics in the case of a time-slice overrun.
 
@@ -147,6 +181,8 @@ then repeatedly after C<repeat> milliseconds.
 B<* Note:> Does not update the event loop's concept of L<UV::Loop/"now">. See
 L<UV::Loop/"update_time"> for more information.
 
+Returns the C<$timer> instance itself.
+
 =head2 stop
 
     $timer->stop();
